==> micronaut-data-runtime-3.9.6-sources.jar: io/micronaut/data/runtime/intercept/async/DefaultFindPageAsyncInterceptor.java <==
/*
 * Copyright 2017-2020 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.micronaut.data.runtime.intercept.async;

import io.micronaut.core.annotation.NonNull;
import io.micronaut.aop.MethodInvocationContext;
import io.micronaut.core.util.CollectionUtils;
import io.micronaut.data.annotation.Query;
import io.micronaut.data.intercept.RepositoryMethodKey;
import io.micronaut.data.operations.RepositoryOperations;
import io.micronaut.data.intercept.async.FindPageAsyncInterceptor;
import io.micronaut.data.model.Page;
import io.micronaut.data.model.runtime.PreparedQuery;
import io.micronaut.transaction.support.TransactionSynchronizationManager;

import java.util.List;
import java.util.concurrent.CompletionStage;

/**
 * Default implementation of {@link FindPageAsyncInterceptor}.
 *
 * @author graemerocher
 * @since 1.0.0
 */
public class DefaultFindPageAsyncInterceptor extends AbstractConvertCompletionStageInterceptor<Page<Object>> implements FindPageAsyncInterceptor<Object> {

    /**
     * Default constructor.
     *
     * @param datastore The operations
     */
    protected DefaultFindPageAsyncInterceptor(@NonNull RepositoryOperations datastore) {
        super(datastore);
    }

    @Override
    protected CompletionStage<?> interceptCompletionStage(RepositoryMethodKey methodKey, MethodInvocationContext<Object, CompletionStage<Page<Object>>> context) {
        if (context.hasAnnotation(Query.class)) {
            PreparedQuery<?, ?> preparedQuery = prepareQuery(methodKey, context);
            PreparedQuery<?, Number> countQuery = prepareCountQuery(methodKey, context);
            TransactionSynchronizationManager.TransactionSynchronizationState state = TransactionSynchronizationManager.getState();
            return asyncDatastoreOperations.findOne(countQuery)
                .thenCompose(total -> {
                    try (TransactionSynchronizationManager.TransactionSynchronizationStateOp ignore = TransactionSynchronizationManager.withState(state)) {
                        return asyncDatastoreOperations.findAll(preparedQuery)
                            .thenApply(objects -> {
                                List<Object> resultList = CollectionUtils.iterableToList((Iterable<Object>) objects);
                                return Page.of(resultList, getPageable(context), total.longValue());
                            });
                    }
                });

        }
        return asyncDatastoreOperations.findPage(getPagedQuery(context));
    }

}

==> micronaut-data-runtime-4.8.0-sources.jar: io/micronaut/data/runtime/intercept/async/DefaultFindPageAsyncInterceptor.java <==
/*
 * Copyright 2017-2020 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package io.micronaut.data.runtime.intercept.async;

import io.micronaut.aop.MethodInvocationContext;
import io.micronaut.core.annotation.NonNull;
import io.micronaut.core.util.CollectionUtils;
import io.micronaut.data.annotation.Query;
import io.micronaut.data.intercept.RepositoryMethodKey;
import io.micronaut.data.intercept.async.FindPageAsyncInterceptor;
import io.micronaut.data.model.Page;
import io.micronaut.data.model.runtime.PreparedQuery;
import io.micronaut.data.operations.RepositoryOperations;

import java.util.List;
import java.util.concurrent.CompletionStage;

/**
 * Default implementation of {@link FindPageAsyncInterceptor}.
 *
 * @author graemerocher
 * @since 1.0.0
 */
public class DefaultFindPageAsyncInterceptor extends AbstractConvertCompletionStageInterceptor<Page<Object>> implements FindPageAsyncInterceptor<Object> {

    /**
     * Default constructor.
     *
     * @param datastore The operations
     */
    protected DefaultFindPageAsyncInterceptor(@NonNull RepositoryOperations datastore) {
        super(datastore);
    }

    @Override
    protected CompletionStage<?> interceptCompletionStage(RepositoryMethodKey methodKey, MethodInvocationContext<Object, CompletionStage<Page<Object>>> context) {
        if (context.hasAnnotation(Query.class)) {
            PreparedQuery<?, ?> preparedQuery = prepareQuery(methodKey, context);
            PreparedQuery<?, Number> countQuery = prepareCountQuery(methodKey, context);
            return asyncDatastoreOperations.findOne(countQuery)
                .thenCompose(total -> asyncDatastoreOperations.findAll(preparedQuery)
                    .thenApply(objects -> {
                        List<Object> resultList = CollectionUtils.iterableToList((Iterable<Object>) objects);
                        return Page.of(resultList, getPageable(context), total.longValue());
                    }));
        }
        return asyncDatastoreOperations.findPage(getPagedQuery(context));
    }

}
